Subject: [PATCH 09/11] dm: snapshot: abstract memory release
To: Andrew Morton <akpm@osdl.org>
Cc: linux-kernel@vger.kernel.org, dm-devel@redhat.com, Milan Broz <mbroz@redhat.com>

From: Milan Broz <mbroz@redhat.com>

Move the code that releases memory used by a snapshot into a separate function.

Signed-off-by: Milan Broz <mbroz@redhat.com>
Signed-off-by: Alasdair G Kergon <agk@redhat.com>
Cc: dm-devel@redhat.com

 drivers/md/dm-snap.c |   19 ++++++++++++-------
 1 files changed, 12 insertions(+), 7 deletions(-)

Index: linux-2.6.19/drivers/md/dm-snap.c
===================================================================
--- linux-2.6.19.orig/drivers/md/dm-snap.c	2006-12-06 20:49:31.000000000 +0000
+++ linux-2.6.19/drivers/md/dm-snap.c	2006-12-06 20:49:33.000000000 +0000
@@ -564,6 +564,17 @@ static int snapshot_ctr(struct dm_target
 	return r;
 }
 
+static void __free_exceptions(struct dm_snapshot *s)
+{
+	kcopyd_client_destroy(s->kcopyd_client);
+	s->kcopyd_client = NULL;
+
+	exit_exception_table(&s->pending, pending_cache);
+	exit_exception_table(&s->complete, exception_cache);
+
+	s->store.destroy(&s->store);
+}
+
 static void snapshot_dtr(struct dm_target *ti)
 {
 	struct dm_snapshot *s = (struct dm_snapshot *) ti->private;
@@ -574,13 +585,7 @@ static void snapshot_dtr(struct dm_targe
 	/* After this returns there can be no new kcopyd jobs. */
 	unregister_snapshot(s);
 
-	kcopyd_client_destroy(s->kcopyd_client);
-
-	exit_exception_table(&s->pending, pending_cache);
-	exit_exception_table(&s->complete, exception_cache);
-
-	/* Deallocate memory used */
-	s->store.destroy(&s->store);
+	__free_exceptions(s);
 
 	dm_put_device(ti, s->origin);
 	dm_put_device(ti, s->cow);
